# ----------------------------------------------------------------------------
#
#  Welcome to Baml! To use this generated code, please run the following:
#
#  $ pip install baml
#
# ----------------------------------------------------------------------------

# This file was generated by BAML: please do not edit it. Instead, edit the
# BAML files and re-generate this code using: baml-cli generate
# baml-cli is available with the baml package.

import typing
from baml_py import type_builder
from baml_py import baml_py
# These are exports, not used here, hence the linter is disabled
from baml_py.baml_py import FieldType, EnumValueBuilder, EnumBuilder, ClassBuilder # noqa: F401 # pylint: disable=unused-import
from .globals import DO_NOT_USE_DIRECTLY_UNLESS_YOU_KNOW_WHAT_YOURE_DOING_RUNTIME

class TypeBuilder(type_builder.TypeBuilder):
    def __init__(self):
        super().__init__(classes=set(
          ["CapabilityCandidate","CapabilityCategory","CapabilityMatchResult","CapabilitySummary","CodeSnippet","ManifestCLI","ManifestServer","ManifestSummary","ToolArg","ToolDescription","ToolInfo",]
        ), enums=set(
          []
        ), runtime=DO_NOT_USE_DIRECTLY_UNLESS_YOU_KNOW_WHAT_YOURE_DOING_RUNTIME)

    # #########################################################################
    # Generated enums 0
    # #########################################################################


    # #########################################################################
    # Generated classes 11
    # #########################################################################

    @property
    def CapabilityCandidate(self) -> "CapabilityCandidateViewer":
        return CapabilityCandidateViewer(self)

    @property
    def CapabilityCategory(self) -> "CapabilityCategoryViewer":
        return CapabilityCategoryViewer(self)

    @property
    def CapabilityMatchResult(self) -> "CapabilityMatchResultViewer":
        return CapabilityMatchResultViewer(self)

    @property
    def CapabilitySummary(self) -> "CapabilitySummaryViewer":
        return CapabilitySummaryViewer(self)

    @property
    def CodeSnippet(self) -> "CodeSnippetViewer":
        return CodeSnippetViewer(self)

    @property
    def ManifestCLI(self) -> "ManifestCLIViewer":
        return ManifestCLIViewer(self)

    @property
    def ManifestServer(self) -> "ManifestServerViewer":
        return ManifestServerViewer(self)

    @property
    def ManifestSummary(self) -> "ManifestSummaryViewer":
        return ManifestSummaryViewer(self)

    @property
    def ToolArg(self) -> "ToolArgViewer":
        return ToolArgViewer(self)

    @property
    def ToolDescription(self) -> "ToolDescriptionViewer":
        return ToolDescriptionViewer(self)

    @property
    def ToolInfo(self) -> "ToolInfoViewer":
        return ToolInfoViewer(self)



# #########################################################################
# Generated enums 0
# #########################################################################


# #########################################################################
# Generated classes 11
# #########################################################################

class CapabilityCandidateAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("CapabilityCandidate")
        self._properties: typing.Set[str] = set([  "name",  "candidate_type",  "relevance_score",  "reasoning",  "requires_api_key",  ])
        self._props = CapabilityCandidateProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "CapabilityCandidateProperties":
        return self._props


class CapabilityCandidateViewer(CapabilityCandidateAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class CapabilityCandidateProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("name"))
    
    @property
    def candidate_type(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("candidate_type"))
    
    @property
    def relevance_score(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("relevance_score"))
    
    @property
    def reasoning(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("reasoning"))
    
    @property
    def requires_api_key(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("requires_api_key"))
    
    


class CapabilityCategoryAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("CapabilityCategory")
        self._properties: typing.Set[str] = set([  "name",  "server",  "summary",  ])
        self._props = CapabilityCategoryProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "CapabilityCategoryProperties":
        return self._props


class CapabilityCategoryViewer(CapabilityCategoryAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class CapabilityCategoryProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("name"))
    
    @property
    def server(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("server"))
    
    @property
    def summary(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("summary"))
    
    


class CapabilityMatchResultAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("CapabilityMatchResult")
        self._properties: typing.Set[str] = set([  "candidates",  "recommendation",  ])
        self._props = CapabilityMatchResultProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "CapabilityMatchResultProperties":
        return self._props


class CapabilityMatchResultViewer(CapabilityMatchResultAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class CapabilityMatchResultProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def candidates(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("candidates"))
    
    @property
    def recommendation(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("recommendation"))
    
    


class CapabilitySummaryAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("CapabilitySummary")
        self._properties: typing.Set[str] = set([  "categories",  "usage_hint",  ])
        self._props = CapabilitySummaryProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "CapabilitySummaryProperties":
        return self._props


class CapabilitySummaryViewer(CapabilitySummaryAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class CapabilitySummaryProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def categories(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("categories"))
    
    @property
    def usage_hint(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("usage_hint"))
    
    


class CodeSnippetAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("CodeSnippet")
        self._properties: typing.Set[str] = set([  "snippet",  "pattern",  ])
        self._props = CodeSnippetProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "CodeSnippetProperties":
        return self._props


class CodeSnippetViewer(CodeSnippetAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class CodeSnippetProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def snippet(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("snippet"))
    
    @property
    def pattern(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("pattern"))
    
    


class ManifestCLIAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("ManifestCLI")
        self._properties: typing.Set[str] = set([  "name",  "description",  "keywords",  ])
        self._props = ManifestCLIProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "ManifestCLIProperties":
        return self._props


class ManifestCLIViewer(ManifestCLIAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class ManifestCLIProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("name"))
    
    @property
    def description(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("description"))
    
    @property
    def keywords(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("keywords"))
    
    


class ManifestServerAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("ManifestServer")
        self._properties: typing.Set[str] = set([  "name",  "description",  "keywords",  "requires_api_key",  "env_var",  ])
        self._props = ManifestServerProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "ManifestServerProperties":
        return self._props


class ManifestServerViewer(ManifestServerAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class ManifestServerProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("name"))
    
    @property
    def description(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("description"))
    
    @property
    def keywords(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("keywords"))
    
    @property
    def requires_api_key(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("requires_api_key"))
    
    @property
    def env_var(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("env_var"))
    
    


class ManifestSummaryAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("ManifestSummary")
        self._properties: typing.Set[str] = set([  "servers",  "clis",  ])
        self._props = ManifestSummaryProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "ManifestSummaryProperties":
        return self._props


class ManifestSummaryViewer(ManifestSummaryAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class ManifestSummaryProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def servers(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("servers"))
    
    @property
    def clis(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("clis"))
    
    


class ToolArgAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("ToolArg")
        self._properties: typing.Set[str] = set([  "name",  "type",  "required",  "description",  ])
        self._props = ToolArgProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "ToolArgProperties":
        return self._props


class ToolArgViewer(ToolArgAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class ToolArgProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("name"))
    
    @property
    def type(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("type"))
    
    @property
    def required(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("required"))
    
    @property
    def description(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("description"))
    
    


class ToolDescriptionAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("ToolDescription")
        self._properties: typing.Set[str] = set([  "server_name",  "tool_name",  "description",  ])
        self._props = ToolDescriptionProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "ToolDescriptionProperties":
        return self._props


class ToolDescriptionViewer(ToolDescriptionAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class ToolDescriptionProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def server_name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("server_name"))
    
    @property
    def tool_name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("tool_name"))
    
    @property
    def description(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("description"))
    
    


class ToolInfoAst:
    def __init__(self, tb: type_builder.TypeBuilder):
        _tb = tb._tb # type: ignore (we know how to use this private attribute)
        self._bldr = _tb.class_("ToolInfo")
        self._properties: typing.Set[str] = set([  "tool_id",  "tool_name",  "description",  "args",  ])
        self._props = ToolInfoProperties(self._bldr, self._properties)

    def type(self) -> baml_py.FieldType:
        return self._bldr.field()

    @property
    def props(self) -> "ToolInfoProperties":
        return self._props


class ToolInfoViewer(ToolInfoAst):
    def __init__(self, tb: type_builder.TypeBuilder):
        super().__init__(tb)

    
    def list_properties(self) -> typing.List[typing.Tuple[str, type_builder.ClassPropertyViewer]]:
        return [(name, type_builder.ClassPropertyViewer(self._bldr.property(name))) for name in self._properties]
    


class ToolInfoProperties:
    def __init__(self, bldr: baml_py.ClassBuilder, properties: typing.Set[str]):
        self.__bldr = bldr
        self.__properties = properties # type: ignore (we know how to use this private attribute) # noqa: F821

    
    
    @property
    def tool_id(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("tool_id"))
    
    @property
    def tool_name(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("tool_name"))
    
    @property
    def description(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("description"))
    
    @property
    def args(self) -> type_builder.ClassPropertyViewer:
        return type_builder.ClassPropertyViewer(self.__bldr.property("args"))
    
    




"""Tests for package version checking functionality."""

from __future__ import annotations

import asyncio
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from pmcp.manifest.version_checker import (
    _version_cache,
    clear_version_cache,
    detect_package_type,
    get_npm_version,
    get_package_version,
    get_pypi_version,
    is_version_newer,
)


class TestDetectPackageType:
    """Tests for detect_package_type function."""

    def test_npx_simple_package(self) -> None:
        """Test detection of simple npx package."""
        pkg_type, pkg_name = detect_package_type("npx", ["-y", "playwright-mcp"])
        assert pkg_type == "npm"
        assert pkg_name == "playwright-mcp"

    def test_npx_scoped_package(self) -> None:
        """Test detection of scoped npm package."""
        pkg_type, pkg_name = detect_package_type("npx", ["-y", "@playwright/mcp"])
        assert pkg_type == "npm"
        assert pkg_name == "@playwright/mcp"

    def test_npx_package_with_latest(self) -> None:
        """Test detection strips @latest suffix."""
        pkg_type, pkg_name = detect_package_type("npx", ["-y", "some-package@latest"])
        assert pkg_type == "npm"
        assert pkg_name == "some-package"

    def test_npx_without_y_flag(self) -> None:
        """Test detection works without -y flag."""
        pkg_type, pkg_name = detect_package_type("npx", ["my-mcp-server"])
        assert pkg_type == "npm"
        assert pkg_name == "my-mcp-server"

    def test_npm_command(self) -> None:
        """Test detection with npm command picks first non-flag arg."""
        # Note: npm exec is treated as package name since code doesn't special-case it
        pkg_type, pkg_name = detect_package_type("npm", ["-y", "server-pkg"])
        assert pkg_type == "npm"
        assert pkg_name == "server-pkg"

    def test_uvx_simple_package(self) -> None:
        """Test detection of uvx (PyPI) package."""
        pkg_type, pkg_name = detect_package_type("uvx", ["mcp-server-git"])
        assert pkg_type == "pypi"
        assert pkg_name == "mcp-server-git"

    def test_uvx_with_flags(self) -> None:
        """Test uvx detection skips flags."""
        pkg_type, pkg_name = detect_package_type(
            "uvx", ["--quiet", "my-package", "--arg"]
        )
        assert pkg_type == "pypi"
        assert pkg_name == "my-package"

    def test_unknown_command(self) -> None:
        """Test unknown command returns unknown type."""
        pkg_type, pkg_name = detect_package_type("python", ["-m", "mymodule"])
        assert pkg_type == "unknown"
        assert pkg_name is None

    def test_docker_command(self) -> None:
        """Test docker command returns unknown."""
        pkg_type, pkg_name = detect_package_type("docker", ["run", "myimage"])
        assert pkg_type == "unknown"
        assert pkg_name is None

    def test_empty_args(self) -> None:
        """Test npx with empty args."""
        pkg_type, pkg_name = detect_package_type("npx", [])
        assert pkg_type == "unknown"
        assert pkg_name is None

    def test_only_flags(self) -> None:
        """Test npx with only flags."""
        pkg_type, pkg_name = detect_package_type("npx", ["-y", "--quiet"])
        assert pkg_type == "unknown"
        assert pkg_name is None


class TestIsVersionNewer:
    """Tests for is_version_newer function."""

    def test_same_version(self) -> None:
        """Test same versions are not newer."""
        assert is_version_newer("1.0.0", "1.0.0") is False
        assert is_version_newer("2025.1.1", "2025.1.1") is False

    def test_semver_patch_newer(self) -> None:
        """Test patch version comparison."""
        assert is_version_newer("1.0.0", "1.0.1") is True
        assert is_version_newer("1.0.1", "1.0.0") is False

    def test_semver_minor_newer(self) -> None:
        """Test minor version comparison."""
        assert is_version_newer("1.0.0", "1.1.0") is True
        assert is_version_newer("1.1.0", "1.0.0") is False

    def test_semver_major_newer(self) -> None:
        """Test major version comparison."""
        assert is_version_newer("1.0.0", "2.0.0") is True
        assert is_version_newer("2.0.0", "1.0.0") is False

    def test_date_based_version(self) -> None:
        """Test date-based version comparison."""
        assert is_version_newer("2025.1.1", "2025.1.2") is True
        assert is_version_newer("2025.1.1", "2025.2.1") is True
        assert is_version_newer("2025.12.1", "2025.1.1") is False

    def test_version_with_v_prefix(self) -> None:
        """Test versions with v prefix."""
        assert is_version_newer("v1.0.0", "v1.0.1") is True
        assert is_version_newer("V1.0.0", "V1.0.1") is True

    def test_short_version(self) -> None:
        """Test 2-part versions."""
        assert is_version_newer("1.0", "1.1") is True
        assert is_version_newer("0.19", "0.20") is True

    def test_different_length_versions(self) -> None:
        """Test versions with different number of parts."""
        # 1.0 vs 1.0.1 - tuple comparison: (1, 0) vs (1, 0, 1)
        assert is_version_newer("1.0", "1.0.1") is True
        assert is_version_newer("1.0.1", "1.0") is False

    def test_zero_versions(self) -> None:
        """Test pre-release style versions."""
        assert is_version_newer("0.0.1", "0.0.2") is True
        assert is_version_newer("0.0.19", "0.0.20") is True

    def test_non_numeric_versions(self) -> None:
        """Test non-numeric versions parse as empty tuples (equal)."""
        # Non-numeric strings have no numeric parts, so both parse as ()
        # () > () is False, so neither is "newer"
        assert is_version_newer("alpha", "beta") is False
        assert is_version_newer("beta", "alpha") is False

    def test_mixed_versions(self) -> None:
        """Test versions with mixed numeric and text parts."""
        # "rc1" parses as (1,), "rc2" parses as (2,)
        assert is_version_newer("1.0.0-rc1", "1.0.0-rc2") is True
        assert is_version_newer("v2.0-beta1", "v2.0-beta2") is True


class TestGetNpmVersion:
    """Tests for get_npm_version function."""

    @pytest.fixture(autouse=True)
    def clear_cache(self) -> None:
        """Clear version cache before each test."""
        clear_version_cache()

    @pytest.mark.asyncio
    async def test_successful_lookup(self) -> None:
        """Test successful npm version lookup."""
        mock_response = MagicMock()
        mock_response.status = 200
        mock_response.json = AsyncMock(return_value={"dist-tags": {"latest": "1.2.3"}})
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("test-package")
            assert version == "1.2.3"

    @pytest.mark.asyncio
    async def test_scoped_package_url_encoding(self) -> None:
        """Test scoped npm package URL encoding."""
        mock_response = MagicMock()
        mock_response.status = 200
        mock_response.json = AsyncMock(return_value={"dist-tags": {"latest": "0.1.0"}})
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("@playwright/mcp")
            assert version == "0.1.0"
            # Check URL was properly encoded
            call_args = mock_session.get.call_args
            url = call_args[0][0]
            assert "%40" in url  # @ encoded
            assert "%2F" in url  # / encoded

    @pytest.mark.asyncio
    async def test_404_response(self) -> None:
        """Test handling of 404 response."""
        mock_response = MagicMock()
        mock_response.status = 404
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("nonexistent-package")
            assert version is None

    @pytest.mark.asyncio
    async def test_timeout_error(self) -> None:
        """Test handling of timeout."""
        mock_session = MagicMock()
        mock_session.get = MagicMock(side_effect=asyncio.TimeoutError())
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("test-package", timeout=0.1)
            assert version is None

    @pytest.mark.asyncio
    async def test_network_error(self) -> None:
        """Test handling of network error."""
        mock_session = MagicMock()
        mock_session.get = MagicMock(side_effect=Exception("Network error"))
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("test-package")
            assert version is None

    @pytest.mark.asyncio
    async def test_missing_dist_tags(self) -> None:
        """Test handling of response missing dist-tags."""
        mock_response = MagicMock()
        mock_response.status = 200
        mock_response.json = AsyncMock(return_value={"name": "test"})
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("test-package")
            assert version is None

    @pytest.mark.asyncio
    async def test_cache_hit(self) -> None:
        """Test that cached versions are returned without network call."""
        # Populate cache
        _version_cache["npm:cached-package"] = "1.0.0"

        # Should not make network call
        mock_session = MagicMock()
        mock_session.get = MagicMock(side_effect=Exception("Should not be called"))

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_npm_version("cached-package")
            assert version == "1.0.0"
            mock_session.get.assert_not_called()


class TestGetPypiVersion:
    """Tests for get_pypi_version function."""

    @pytest.fixture(autouse=True)
    def clear_cache(self) -> None:
        """Clear version cache before each test."""
        clear_version_cache()

    @pytest.mark.asyncio
    async def test_successful_lookup(self) -> None:
        """Test successful PyPI version lookup."""
        mock_response = MagicMock()
        mock_response.status = 200
        mock_response.json = AsyncMock(return_value={"info": {"version": "2025.12.18"}})
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_pypi_version("mcp-server-git")
            assert version == "2025.12.18"

    @pytest.mark.asyncio
    async def test_correct_url(self) -> None:
        """Test PyPI URL is correct."""
        mock_response = MagicMock()
        mock_response.status = 200
        mock_response.json = AsyncMock(return_value={"info": {"version": "1.0.0"}})
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            await get_pypi_version("my-package")
            call_args = mock_session.get.call_args
            url = call_args[0][0]
            assert url == "https://pypi.org/pypi/my-package/json"

    @pytest.mark.asyncio
    async def test_404_response(self) -> None:
        """Test handling of 404 response."""
        mock_response = MagicMock()
        mock_response.status = 404
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_pypi_version("nonexistent-package")
            assert version is None

    @pytest.mark.asyncio
    async def test_timeout_error(self) -> None:
        """Test handling of timeout."""
        mock_session = MagicMock()
        mock_session.get = MagicMock(side_effect=asyncio.TimeoutError())
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_pypi_version("test-package")
            assert version is None

    @pytest.mark.asyncio
    async def test_missing_info(self) -> None:
        """Test handling of response missing info."""
        mock_response = MagicMock()
        mock_response.status = 200
        mock_response.json = AsyncMock(return_value={"name": "test"})
        mock_response.__aenter__ = AsyncMock(return_value=mock_response)
        mock_response.__aexit__ = AsyncMock(return_value=None)

        mock_session = MagicMock()
        mock_session.get = MagicMock(return_value=mock_response)
        mock_session.__aenter__ = AsyncMock(return_value=mock_session)
        mock_session.__aexit__ = AsyncMock(return_value=None)

        with patch("aiohttp.ClientSession", return_value=mock_session):
            version = await get_pypi_version("test-package")
            assert version is None

    @pytest.mark.asyncio
    async def test_cache_hit(self) -> None:
        """Test that cached versions are returned without network call."""
        _version_cache["pypi:cached-package"] = "2.0.0"

        version = await get_pypi_version("cached-package")
        assert version == "2.0.0"


class TestGetPackageVersion:
    """Tests for get_package_version function."""

    @pytest.fixture(autouse=True)
    def clear_cache(self) -> None:
        """Clear version cache before each test."""
        clear_version_cache()

    @pytest.mark.asyncio
    async def test_npm_package(self) -> None:
        """Test npm package version lookup."""
        with patch(
            "pmcp.manifest.version_checker.get_npm_version",
            new_callable=AsyncMock,
            return_value="1.0.0",
        ):
            version, pkg_type = await get_package_version("npx", ["-y", "my-package"])
            assert version == "1.0.0"
            assert pkg_type == "npm"

    @pytest.mark.asyncio
    async def test_pypi_package(self) -> None:
        """Test PyPI package version lookup."""
        with patch(
            "pmcp.manifest.version_checker.get_pypi_version",
            new_callable=AsyncMock,
            return_value="2.0.0",
        ):
            version, pkg_type = await get_package_version("uvx", ["my-package"])
            assert version == "2.0.0"
            assert pkg_type == "pypi"

    @pytest.mark.asyncio
    async def test_unknown_package(self) -> None:
        """Test unknown package type."""
        version, pkg_type = await get_package_version("docker", ["run", "image"])
        assert version is None
        assert pkg_type == "unknown"

    @pytest.mark.asyncio
    async def test_npm_lookup_failure(self) -> None:
        """Test handling of npm lookup failure."""
        with patch(
            "pmcp.manifest.version_checker.get_npm_version",
            new_callable=AsyncMock,
            return_value=None,
        ):
            version, pkg_type = await get_package_version("npx", ["-y", "my-package"])
            assert version is None
            assert pkg_type == "npm"


class TestClearVersionCache:
    """Tests for clear_version_cache function."""

    def test_clears_cache(self) -> None:
        """Test cache is cleared."""
        _version_cache["npm:test"] = "1.0.0"
        _version_cache["pypi:test"] = "2.0.0"
        assert len(_version_cache) == 2

        clear_version_cache()
        assert len(_version_cache) == 0



"""LLM-powered capability summary generation using BAML.

Uses BAML for type-safe prompt management with Anthropic Claude.
"""

from __future__ import annotations

import asyncio
import logging
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pmcp.types import ToolInfo

logger = logging.getLogger(__name__)

# Timeout for LLM summarization (seconds)
SUMMARIZE_TIMEOUT = 30


async def summarize_capabilities(tools: list[ToolInfo]) -> str:
    """Use BAML to generate a capability summary.

    Args:
        tools: List of tools to summarize

    Returns:
        Human-readable capability summary

    Raises:
        ImportError: If baml-py is not installed
        TimeoutError: If summarization takes too long
        Exception: If BAML call fails
    """
    # Import here to make dependency optional
    try:
        from pmcp.baml_client import b
        from pmcp.baml_client.types import ToolDescription
    except ImportError as e:
        raise ImportError(
            "baml-py not installed. Install with: pip install baml-py"
        ) from e

    if not tools:
        return "MCP Gateway: No tools available."

    # Convert to BAML types
    tool_descriptions = [
        ToolDescription(
            server_name=t.server_name,
            tool_name=t.tool_name,
            description=t.short_description,
        )
        for t in tools
    ]

    logger.debug("Requesting BAML summary for %d tools", len(tools))

    try:
        async with asyncio.timeout(SUMMARIZE_TIMEOUT):
            result = await b.SummarizeCapabilities(tool_descriptions)
    except TimeoutError:
        logger.warning("BAML summarization timed out after %ds", SUMMARIZE_TIMEOUT)
        raise

    # Format as string
    lines = ["MCP Gateway capabilities:"]
    for cat in result.categories:
        lines.append(f"• {cat.name} ({cat.server}): {cat.summary}")
    lines.append(f"\n{result.usage_hint}")

    summary = "\n".join(lines)
    logger.debug("Generated BAML summary: %d chars", len(summary))
    return summary
